/*
    Copyright  1995-2001, The AROS Development Team. All rights reserved.
    $Id: locale_init.c,v 1.16 2018/02/25 22:55:19 piru Exp $

    Desc: Initialisation for the locale.library.
    Lang: english
*/

#define AROS_ALMOST_COMPATIBLE
#include <exec/types.h>
#include <exec/lists.h>
#include <exec/memory.h>
#include <exec/resident.h>
#include <aros/libcall.h>
#include <exec/semaphores.h>
#include <aros/asmcall.h>

#include <proto/exec.h>
#include <proto/dos.h>

#include "locale_intern.h"
#include "libdefs.h"

/*
    Why don't I use C_Lib, simple, it doesn't seem to be able to do
    non-expunge libraries, which locale is.
*/

#ifdef __MORPHOS__
    const unsigned long __abox__ __attribute__((section(".rodata"))) = 1;
#endif

#define INIT    AROS_SLIB_ENTRY(init, Locale)
static const char name[];
static const char version[];
static const APTR inittabl[4];
static const void * const LIBFUNCTABLE[];
LIBBASETYPE * INIT();
extern const char LIBEND;

extern void SetLocaleLanguage(struct IntLocale *, struct LocaleBase *);

const struct Resident __TEXTSEGMENT__ Locale_resident =
{
    RTC_MATCHWORD,
    (struct Resident *)&Locale_resident,
    (struct Resident *)&Locale_resident+1,
#ifdef __MORPHOS__
    RTF_PPC | RTF_EXTENDED | RTF_AUTOINIT,
#else
    RTF_AUTOINIT,
#endif
    VERSION_NUMBER,
    NT_LIBRARY,
    -120,
    (char *)name,
    (char *)&version[6],
    (ULONG *)inittabl
#ifdef __MORPHOS__
    ,
    REVISION_NUMBER,	/* Revision */
    NULL /* Tags */
#endif
};

static const char name[]=NAME_STRING;
static const char __TEXTSEGMENT__ version[]=VERSION_STRING;

static const APTR inittabl[4] =
{
    (APTR)sizeof(struct IntLocaleBase),
    (APTR)LIBFUNCTABLE,
    NULL,
    &INIT
};
#undef DOSBase
#undef UtilityBase
#undef IFFParseBase

/* This global variable is needed for LocRawDoFmt */

struct LocaleBase *globallocalebase;
#include <exec/execbase.h>

#ifdef __MORPHOS__
void install_timezone_patches(struct LocaleBase *LocaleBase);
#endif

static
void cleanupbase(LIBBASETYPE *LIBBASE)
{
  FreeMem(((UBYTE *) LIBBASE) - LIBBASE->lb_LibNode.lib_NegSize,
          LIBBASE->lb_LibNode.lib_NegSize + LIBBASE->lb_LibNode.lib_PosSize);
}


#ifdef __MORPHOS__
LIBBASETYPE *LIB_init(LIBBASETYPE *LIBBASE, BPTR segList, struct ExecBase *sysBase)
#else
AROS_UFH3(LIBBASETYPE *, AROS_SLIB_ENTRY(init,Locale),
    AROS_UFHA(LIBBASETYPE *,  LIBBASE, D0),
    AROS_UFHA(BPTR,                  segList, A0),
    AROS_UFHA(struct ExecBase *, sysBase, A6)
)
#endif
{
#ifndef __MORPHOS__
    AROS_USERFUNC_INIT
#endif

    struct IntLocale *def;

    SysBase = sysBase;
  
    globallocalebase = LIBBASE;
    
    /* Do whatever static initialisation you need here */
    InitSemaphore(&((struct IntLocaleBase *)LIBBASE)->lb_LocaleLock);
    InitSemaphore(&((struct IntLocaleBase *)LIBBASE)->lb_CatalogLock);
    InitSemaphore(&((struct IntLocaleBase *)LIBBASE)->lb_UnicodeLock);

    NEWLIST(&((struct IntLocaleBase *)LIBBASE)->lb_CatalogList);

    /* We have to open some libraries. */
	if ((IntLB(LIBBASE)->lb_DosBase = (struct DosLibrary *)OpenLibrary("dos.library", 37L)))
	if ((IntLB(LIBBASE)->lb_UtilityBase = OpenLibrary("utility.library", 37L)))
	if ((IntLB(LIBBASE)->lb_IFFParseBase = OpenLibrary("iffparse.library", 37L)))
	if ((IntLB(LIBBASE)->lb_HashTableBase = OpenLibrary("hashtable.library", 50L)))
	{
		IntLB(LIBBASE)->lb_RexxSysBase = OpenLibrary("rexxsyslib.library", 36L);
		IntLB(LIBBASE)->lb_DefaultLocale = def = AllocMem(sizeof(struct IntLocale), MEMF_CLEAR|MEMF_ANY);

		if(def != NULL)
		{
#ifdef __MORPHOS__
			{
				#undef SysBase
				#undef DOSBase
				extern struct ExecBase *SysBase;
				extern struct DosLibrary *DOSBase;
				extern struct SignalSemaphore timezonelock;

				/* Make sure things are set up before we could call
				   TZCheckSysTimeZone or other TZ things. - Piru */
				SysBase = ((struct IntLocaleBase *)LocaleBase)->lb_SysBase;
				DOSBase = ((struct IntLocaleBase *)LocaleBase)->lb_DosBase;
				#define SysBase (((struct IntLocaleBase *)LocaleBase)->lb_SysBase)
				#define DOSBase (((struct IntLocaleBase *)LocaleBase)->lb_DosBase)
				InitSemaphore(&timezonelock);
			}
#endif
			/* Copy the defaults to our new structure */
			CopyMem((APTR) &defLocale, def, sizeof(struct Locale));

			/* Set lb_CurrentLocale *BEFORE* SetLocaleLanguage */
			IntLB(LIBBASE)->lb_CurrentLocale = def;

			/* Setup the languages - will not fail here. */
			SetLocaleLanguage(def, LIBBASE);

			def->il_Count = 0;
			InstallPatches();

#ifdef __MORPHOS__
			install_timezone_patches(LocaleBase);
#endif
			return LIBBASE;
		}
	}

	CloseLibrary((struct Library *)IntLB(LIBBASE)->lb_DosBase);
	CloseLibrary(IntLB(LIBBASE)->lb_UtilityBase);
	CloseLibrary(IntLB(LIBBASE)->lb_IFFParseBase);
	CloseLibrary(IntLB(LIBBASE)->lb_HashTableBase);
	CloseLibrary(IntLB(LIBBASE)->lb_RexxSysBase);
	cleanupbase(LIBBASE);
	return NULL;

#ifndef __MORPHOS__
    AROS_USERFUNC_EXIT
#endif
}

AROS_LH1(LIBBASETYPE *, open,
    AROS_LHA(ULONG, version, D0),
    LIBBASETYPE *, LIBBASE, 1, Locale)
{
    AROS_LIBFUNC_INIT

    /* keep compiler happy */
    version = 0;


    /* What else do we have to do? */
    LIBBASE->lb_LibNode.lib_OpenCnt++;
    LIBBASE->lb_LibNode.lib_Flags &= ~LIBF_DELEXP;

    return LIBBASE;
    AROS_LIBFUNC_EXIT
}

AROS_LH0(BPTR, expunge,
    LIBBASETYPE *, LIBBASE, 3, Locale)
{
    AROS_LIBFUNC_INIT

    /* As I said above, we cannot remove ourselves. */
    LIBBASE->lb_LibNode.lib_Flags &= ~LIBF_DELEXP;

    /* Free some memory if possible */

    flush_collation_data(LIBBASE);
    flush_decomposition_data(LIBBASE);

    return 0;
    AROS_LIBFUNC_EXIT
}

AROS_LH0(BPTR, close,
    LIBBASETYPE *, LIBBASE, 2, Locale)
{
    AROS_LIBFUNC_INIT

    --LIBBASE->lb_LibNode.lib_OpenCnt;

    /*
	We can never exit because of the system patches,
	But we can try and free some memory.
    */
    AROS_LC0(BPTR, expunge, LIBBASETYPE *, LIBBASE, 3, Locale);

    return 0;
    AROS_LIBFUNC_EXIT
}

AROS_LH0I(int, null,
    LIBBASETYPE *, LIBBASE, 4, Locale)
{
    AROS_LIBFUNC_INIT

    return 0;

    AROS_LIBFUNC_EXIT
}
